/**
 * A class describing a physical particle in a 3D space. Particles have a location (x,y, z), a velocity, a maxSpeed,
 * a maxForce,  an acceleration and a mass. 
 *  The particle can be attracted or pushed away from the planets
 */

class Particle {

  //the location, the velocity and the acceleration of the particle
  PVector location;
  PVector velocity;
  PVector acceleration;

  //the mass of the particle
  float mass;

  //the maximum speed of the particle
  float maxSpeed = 2; 

  //the maximum force that can be applied on the particles
  float maxForce = 1;

  //Constructor. ParticleParent can be created with a mass, and a x, y, z location
  Particle(float mass_, float x, float y, float z) {

    //asigns values to data and assign new Pvector to data
    acceleration = new PVector(0, 0, 0);
    velocity = PVector.random3D();
    location = new PVector(x, y, z);
    mass = mass_;
  }

  //this function applies a certain force on the particle
  void applyForce(PVector force) {

    //gets a copy of the force vector
    PVector f = force.get();

    //divides the force by the mass
    f.div(mass);   

    //ad the force to the acceleration
    acceleration.add(f);
  }

  //this function call the update() and display() function at the same time
  void run() {
    update();
    display();
  }

  // Method to update the position of the particle
  void update() {

    //acceleration modifies the velocity and velocity modifies the location of the Particle
    velocity.add(acceleration);
    location.add(velocity);

    //clears the accelareation each time the update() function is called
    acceleration.mult(0);
  }

  // This function displays the particle
  void display() {

    //sets no stroke 
    noStroke();

    //translates the location of the particle according to location.x, location.y and location.z and draw the sphere particle
    pushMatrix();
    translate(location.x, location.y, location.z);
    sphere(mass*2);
    popMatrix();
  }

  //A function that seeks the particle through a target(the planet)
  void seek(PVector target) {

    //a vector pointing from the particle location to the target
    PVector desired = PVector.sub(target, location);  

    //distance between the particle and the target
    float distance = desired.mag();

    //set the magnitude to maxSpeed
    desired.setMag(maxSpeed);

    // Steering = Desired minus Velocity
    PVector steer = PVector.sub(desired, velocity);

    // Limits to maximum steering force
    steer.limit(maxForce);  

    //applies the steering force
    applyForce(steer);
  }

  //this function makes the partcle go away of a target (the planet)
  void goAway(PVector target, float planetMass) {

    //specifies how close is too close of the target
    float desiredSeparation = planetMass + 10 ;

    //distance between the particle and the target
    float distance = PVector.dist(location, target);

    //if distance is smaller than the desired separation
    if (distance < desiredSeparation) {

      //a vector pointing away from the planet location
      PVector difference = PVector.sub(location, target);

      //sets the magnitude to maxSpeed
      difference.setMag(maxSpeed);

      // Steering = Desired minus Velocity
      PVector steer = PVector.sub(difference, velocity);

      //limits to maximum steering force
      steer.limit(maxForce);

      //applies the force
      applyForce(steer);
    }
  }
}