/**
 * A subclass of CelestialObject describing a physical planet in a 3D space. The planet has an arrayList of systems, two angles, a speed orbit,
 * a speed rotation, a gravitational field, a radius and a specific particle color
 */
class Planet extends CelestialObject {

  //arraylist of systems
  ArrayList<ParticleSystem> systems;

  //angle the planet rotates around itself
  float angle2;

  //angle the planet rotates around the sun
  float angle;

  //the speed of the rotation around the sun
  float speedOrbit;

  //the speed of the rotation arount itself
  float speedRotation;

  //distance between the sun and the planet
  float radius;

  //gravitational field of the planet
  float gravitationalField;

  //color to fill the particle system
  color colorParticle;

  //the name of the planet
  String planetName;

  //tells if the planets names are displayed or not
  boolean displayPlanetNames = false;

  //Constructor. PlanetParent can be created with a x, y, z location, a size, a texture, color, 
  //radius, speed orbit, speed rotation, gravitational field and a name
  Planet(PVector location_, float size_, String textureFileName_, color colorParticle_, float radius_, float speedOrbit_, 
    float speedRotation_, float gravitationalField_, String planetName_) {

    //calls the contructor of the parent class
    super(location_, size_, textureFileName_);

    //Assign values to data and new PVector to data
    gravitationalField = gravitationalField_;
    colorParticle = colorParticle_;
    radius = radius_;
    speedOrbit = speedOrbit_;
    speedRotation = speedRotation_;
    planetName = planetName_;
    angle = 0;
    angle2 = 0;

    //creates the system arrayList
    systems = new ArrayList<ParticleSystem>();

    //adds a new system in the arraylist
    systems.add(new ParticleSystem(new PVector(location.x, location.y, location.z)));
  }

  //this function displays the planet
  void display() {

    pushMatrix();

    //translates the location of the planet according to location.x, location.y and location.z and draw the sphere planet
    //need to have a different translate for the sun and planete, otherwise, the particle will be attracted to the sun...
    translate(location.x, location.y, location.z);

    //if displayPlanetNames is true
    if (displayPlanetNames == true) {

      //sets the fill, the font and the size of the text
      fill(35, 232, 174);
      textFont(font);
      textSize(100);

      //displays the name
      text(planetName, 0, -200, 0);
    }

    //for each particle systems
    for (ParticleSystem ps : systems) {

      //fills the particle system with the associated color and a random alpha channel
      fill(colorParticle, random(150, 255));

      //calls the run function
      ps.run(location, mass, gravitationalField);
    }

    //rotate the planet around itself
    rotateY(angle2);

    //sets no fill
    noFill();

    //calls the display function of the parent class
    super.display();
    popMatrix();
  }

  //this function updates the position of the system
  void update() {

    //updates the angle at which the planet is rotating
    float angle = TWO_PI * millis()/speedOrbit;

    //sets the x and z components updated by the angle and multiply by the radius
    location.x = cos(angle) * radius;
    location.z = sin(angle) * radius;

    //updates the angle2
    angle2 += (speedRotation/100);
  }

  //sets a new speed orbit
  void setSpeedOrbit(float newSpeedOrbit) {
    speedOrbit = newSpeedOrbit;
  }

  //sets a new speed rotation
  void setSpeedRotation(float newSpeedRotation) {
    speedRotation = newSpeedRotation;
  }

  //sets a new radius
  void setRadius(float newRadius) {
    radius = newRadius;
  }

  //sets a new gravitational field
  void setGravitationalField(float newGravitationalField) {
    gravitationalField = newGravitationalField;
  }

  //sets a new planet name
  void setPlanetName(String newPlanetName) {
    planetName = newPlanetName;
  }

  //sets a new state to the boolean displayPlanetNames
  void setStateDisplayPlanetNames(boolean newState) {
    displayPlanetNames = newState;
  }

  //this function adds new particle systems to the arraylist systems 
  void addParticleSystem(float mx, float my) { 

    //need to delete planetLocation and sunLocation since we translate it toward this and we want to come back to the orginal location
    systems.add(new ParticleSystem(new PVector(mx - (location.x + width/2), (my - (location.y + height/2)))));
  }

  //this function adds a new particle system when the key "+" is pressed
  void keyPressed() {
    if (key == '+') {
      addParticleSystem(mouseX + random(-100, 100), mouseY + random(-100, 100));
    }
  }
}