/**
 * Title: Project Final- Space Explore
 * Name: Maëlle Morin
 * Date: 19-04-2016
 * Description: Create a prototype for the final project where almost all the functionalities of your code is there. 
 * The prototype needs to use the 4 OOP principles: Abstraction, Encapsulation, Inheritence and polymorphism. 
 * Space Explore is a space simulation that allows user to navigate into a 3D world, to interact with it and to customize it. 
 * Many planets are rotating around a sun and many particles are attracted by planets. A user interface alows the player to customize the 
 * physics values of the planets.
 
 
 * Keys used to navigate in the Space World : all the arrow keys
 * key used to add particles: "+"
 
 * Copyright (C) <2015>  <Maelle Morin>
 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//imports the external library  controlP5
import controlP5.*;

//imports the external library minim
import ddf.minim.*;

//initializes the external library
Minim minim;

//delcares the audio player that will play the background music of the game
AudioPlayer ambientMusic;

//declares the universe
Universe universe;

//declares the control panel
ControlPanel controlPanel;

//declares a start menu
StartMenu myMenu;

//tells if the game starts or not
boolean startGame = false;

//initialize a font
PFont font;

//this function initializes environment
void setup() {

  //sets noStroke and the size of the display, giving a P3D render mode 
  noStroke();
  smooth();
  size(1280, 800, P3D);

  //instatiates the univer, control panel and starting menu
  universe = new Universe();
  controlPanel = new ControlPanel(this, universe);
  myMenu = new StartMenu();

  //initializes the minim instance
  minim = new Minim(this);

  //initializes the ambient music
  //Music by Symbiants under the Licence Creative Commons
  //https://www.jamendo.com/track/1250957/spacex
  ambientMusic = minim.loadFile("Symbiants_-_SpaceX.mp3");

  //adds and displays controller of controlPanel
  controlPanel.addControlPanel();

  //creates font
  font = createFont("VCR_OSD_MONO_1.001.ttf", 12);
}

//this function draws the animation
void draw() {

  //resizes the background image
  universe.getBackground().resize(width, height);

  //resets the background 
  background(universe.getBackground());

  //if the game is not started
  if (startGame == false) {

    //displays the start menu
    myMenu.display();
  }

  //if the game is started
  if (startGame == true) {

    pushMatrix();
    //translates the systems so the planets will turn around the sun
    translate(width/2, height/2, 0);

    //applies a light function on the 3D objects
    lights();

    //applies a blue and white pointlights
    pointLight(0, 0, 255, 10, 0, 100);
    pointLight(30, 30, 30, 100, 100, 100);

    //calls the display function of the universe
    universe.display();

    popMatrix();

    //calls the collision function of the universe
    universe.collision();

    //displays the spaceship's cockpit
    controlPanel.displayCockpit();

    //displays the position of the spaceship
    controlPanel.spaceshipPosition(universe.myCamera.getXLocation(), universe.myCamera.getZLocation());
  }
}

////This function return what's in it when a key is pressed
void keyPressed() {

  //calls the keyPressed function of the camera and controlPanel
  universe.myCamera.keyPressed();
  controlPanel.keyPressed();

  //if the s key is pressed
  if (key == 's') {

    //startGame becomes true
    startGame = true;

    //plays the ambient music and loops it 10 times
    ambientMusic.play();
    ambientMusic.loop(10);
  }
}

//this function calls the keyReleased function of the camera
void keyReleased() {
  universe.myCamera.keyReleased();
}